
import { createRoot } from 'react-dom/client'
import { Suspense } from 'react';
import './index.css'

// Regular import instead of lazy loading the main App component
import App from './App.tsx';

// Componente de fallback enquanto o App não carrega
const LoadingFallback = () => (
  <div className="h-screen w-screen flex items-center justify-center bg-background">
    <div className="text-primary font-display font-bold text-xl animate-pulse">
      Carregando...
    </div>
  </div>
);

// Performance metrics tracking
const reportWebVitals = () => {
  if ('performance' in window && 'getEntriesByType' in performance) {
    const navigation = performance.getEntriesByType('navigation')[0] as PerformanceNavigationTiming;
    if (navigation) {
      // Reporting First Contentful Paint
      const paintMetrics = performance.getEntriesByType('paint');
      const fcp = paintMetrics.find(entry => entry.name === 'first-contentful-paint');
      if (fcp) {
        console.log(`FCP: ${fcp.startTime.toFixed(0)}ms`);
      }
      
      // Reporting DOM Content Loaded
      console.log(`DCL: ${navigation.domContentLoadedEventEnd.toFixed(0)}ms`);
      
      // Reporting Load Complete
      console.log(`Load: ${navigation.loadEventEnd.toFixed(0)}ms`);
    }
  }
};

// Add performance entries observer for Largest Contentful Paint
const observeLCP = () => {
  if (typeof PerformanceObserver === 'function') {
    const lcpObserver = new PerformanceObserver((entryList) => {
      const entries = entryList.getEntries();
      const lastEntry = entries[entries.length - 1];
      
      // TypeScript fix: Cast the entry to any to access LargestContentfulPaint properties
      // LargestContentfulPaint is not well-typed in all TypeScript environments
      const lcpEntry = lastEntry as any;
      
      console.log(`LCP: ${lcpEntry.startTime.toFixed(0)}ms - Element: `, 
        lcpEntry.element ? (document.querySelector(`#${lcpEntry.element.id}`) || lcpEntry.element || 'Unknown element') : 'Unknown element');
    });
    
    lcpObserver.observe({ type: 'largest-contentful-paint', buffered: true });
  }
};

// Patch window.postMessage para prevenir erros de origem
const patchPostMessage = () => {
  // Guardar a implementação original
  const originalPostMessage = window.postMessage;
  
  // Substituir com uma versão mais segura
  window.postMessage = function(message: any, targetOrigin: string, transfer?: Transferable[]) {
    // Verificar se a origem é válida
    if (targetOrigin === 'null' || !targetOrigin) {
      console.warn('Intercepted postMessage with null origin, using * instead');
      targetOrigin = '*';
    }
    
    // Chamar a implementação original com os parâmetros ajustados
    return originalPostMessage.call(this, message, targetOrigin, transfer);
  } as typeof window.postMessage;
  
  console.log('PostMessage patched for safety');
};

// Load critical resources first - Optimized for LCP
const preloadCriticalResources = () => {
  // Preload critical images with higher priority
  const criticalImages = [
    '/lovable-uploads/b180f4bc-a044-4936-bd90-8c5ce2db1e81.png', // Logo
    '/lovable-uploads/aa253962-7383-4ee8-bc4f-593e7abd8722.png'  // Hero image
  ];
  
  criticalImages.forEach(src => {
    const link = document.createElement('link');
    link.rel = 'preload';
    link.as = 'image';
    link.href = src;
    link.crossOrigin = 'anonymous';
    link.fetchPriority = 'high'; // Explicitly set high priority for critical images
    document.head.appendChild(link);
    
    // Additionally, preload the image in memory
    const img = new Image();
    img.src = src;
    img.decode().catch(() => {}); // Decode the image in advance
  });
  
  // Preload critical fonts
  const criticalFonts = [
    'https://fonts.gstatic.com/s/playfairdisplay/v30/nuFvD-vYSZviVYUb_rj3ij__anPXJzDwcbmjWBN2PKdFvXDZbtM.woff2',
    'https://fonts.gstatic.com/s/inter/v12/UcC73FwrK3iLTeHuS_fvQtMwCp50KnMa1ZL7.woff2'
  ];
  
  criticalFonts.forEach(src => {
    const link = document.createElement('link');
    link.rel = 'preload';
    link.as = 'font';
    link.href = src;
    link.type = 'font/woff2';
    link.crossOrigin = 'anonymous';
    document.head.appendChild(link);
  });
};

// Initialize performance measurements
if (process.env.NODE_ENV !== 'production') {
  observeLCP();
}

// Preload critical resources immediately
if (typeof document !== 'undefined') {
  // Execute preload before anything else
  preloadCriticalResources();
}

// Aplicar patch para postMessage
patchPostMessage();

// Renderização com Suspense para melhorar a experiência de carregamento
createRoot(document.getElementById("root")!).render(
  <Suspense fallback={<LoadingFallback />}>
    <App />
  </Suspense>
);

// Optimize idle callbacks for performance
if (typeof window !== 'undefined') {
  // Use a more reliable approach for idle callbacks
  const scheduleIdleTask = (callback: () => void, timeout = 2000) => {
    if ('requestIdleCallback' in window) {
      (window as any).requestIdleCallback(callback, { timeout });
    } else {
      setTimeout(callback, 50); // Small fallback timeout for browsers without requestIdleCallback
    }
  };
  
  // Schedule non-critical tasks
  scheduleIdleTask(() => {
    // Pre-fetch components that might be needed soon
    const preloadComponentsPromises = [
      import('./pages/Index').catch(() => {}),
      import('./components/HeroSection').catch(() => {}),
      import('./components/AboutSection').catch(() => {})
    ];
    
    // Track vitals when browser is idle
    if (process.env.NODE_ENV !== 'production') {
      reportWebVitals();
    }
  });
}
